import { Action } from '../display/action.js';
import {
  Message, PostMessage, PeerMessage, messageFromJSON,
} from './message.js';
import { NetworkNode } from './network-node.js';
import { randomID } from '../../util/helpers.js';

export class ForumNode extends NetworkNode {
  constructor(name, scene) {
    super(name, scene);
    this.actions = {
      ...this.actions,
      storePost: new Action('store post', scene),
    };
  }

  // Process a message from the queue
  async processMessage(messageJson) {
    try {
      await Message.verify(messageJson);
    } catch (e) {
      await this.actions.processMessage.log(this, this, 'invalid signature', null, '-x');
      console.log(`${this.name}: received message with invalid signature`);
      return;
    }

    const { publicKey } = messageJson;
    const message = messageFromJSON(messageJson);

    if (message instanceof PostMessage) {
      await this.processPostMessage(publicKey, message.content);
    } else if (message instanceof PeerMessage) {
      await this.processPeerMessage(publicKey, message.content);
    } else {
      // Unknown message type
      // Penalize sender for wasting our time
    }
  }

  // Process an incoming post, received by whatever means
  async processPost(authorId, post) {
    if (!post.id) {
      post.id = randomID();
    }
    await this.actions.storePost.log(this, this);
    // this.forumView.addPost(authorId, post.id, post, stake);
  }

  // Process a post we received in a message
  async processPostMessage(authorId, { post, stake }) {
    this.processPost(authorId, post, stake);
    await this.broadcast(
      new PeerMessage({
        posts: [{ authorId, post, stake }],
      }),
    );
  }

  // Process a message we receive from a peer
  async processPeerMessage(peerId, { posts }) {
    // We are trusting that the peer verified the signatures of the posts they're forwarding.
    // We could instead have the peer forward the signed messages and re-verify them.
    for (const { authorId, post, stake } of posts) {
      this.processPost(authorId, post, stake);
    }
  }
}
